#import <math.h>
#import "ConeView.h"

@implementation ConeView

#define PI 3.1415926535

- (void)GLinit:sender
{
    float camera_los[3];
    float cam_theta,cam_phi;
    float cam_norm;
    
    GLfloat light_ambient[] =   {0.0, 0.0, 0.0, 1.0};
    GLfloat light_diffuse[] =   {1.0, 1.0, 1.0, 1.0};
    GLfloat light_specular[] =  {1.0, 1.0, 1.0, 1.0};
    GLfloat light_position[] =  {1.0, 1.0, 1.0, 0.0};

    angle = 0.0;
    spinangle = 0.0;

    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_DEPTH_TEST);

    camera_roll = 90.0*PI/180.;

    camera_eye[0] = -20.0;
    camera_eye[1] = 0.0;
    camera_eye[2] = 50.0;

    camera_view[0] = 0.0;
    camera_view[1] = 0.0;
    camera_view[2] = 0.0;

    camera_los[0] = camera_eye[0] - camera_view[0];
    camera_los[1] = camera_eye[1] - camera_view[1];
    camera_los[2] = camera_eye[2] - camera_view[2];

    cam_norm = sqrt(camera_los[0]*camera_los[0] +
		    camera_los[1]*camera_los[1] +
		    camera_los[2]*camera_los[2]);

    if(cam_norm > 1.0e-10) {
	camera_los[0] /= cam_norm;
	camera_los[1] /= cam_norm;
	camera_los[2] /= cam_norm;

	cam_theta = acos(camera_los[2]);
	cam_phi = atan2(camera_los[1],camera_los[0]);
    }
    else {
	cam_theta = 0.0;
	cam_phi = 0.0;
    }

    glMatrixMode(GL_PROJECTION);
    glRotatef(-90.0, 0.0, 0.0, 1.0);
    glRotatef(-cam_theta*180.0/PI, 0.0, 1.0, 0.0);
    glRotatef(-cam_phi*180./PI, 0.0, 0.0, 1.0);
    glRotatef(camera_roll*180./PI,camera_los[0],camera_los[1],camera_los[2]);
    glTranslatef(-camera_eye[0], -camera_eye[1], -camera_eye[2]);


    glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);

#if 1
    timer = [NSTimer scheduledTimerWithTimeInterval:(NSTimeInterval)0.025
	             target:self selector:@selector(display)
	             userInfo:nil repeats:YES];
#endif    
    return;
}

- (void)GLdraw:sender
{
    void makeFrustum(float, float, float, float, float);
    void makeCap(float, float, float, float);

    float rbasef1 = 5.0;
    float rcapf1  = 4.0;
    float heightf1 = 7.0;
    float rbasef2 = rcapf1;
    float rcapf2  = 2.0;
    float heightf2 = 4.0;
    float phi1 = 5.0*3.14159/180.0;
    float phi2 = 355.0*3.14159/180.0;
    float x,y,z;
    float alphaf2 = atan((rbasef2-rcapf2)/heightf2);

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* Lets try rotating the viewpoint of the camera */

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    x = cos(angle);
    y = sin(angle);
    z = 0.0;

#if 1
    glRotatef(10.0, x,y,z);
    glRotatef(spinangle, 0.0, 0.0, 1.0);
#endif

    glColor3f(1.0, 0.0, 0.0);
    makeFrustum(rbasef1,rcapf1,heightf1,phi1,phi2);

    glTranslatef(0.0, 0.0, heightf1);
    makeFrustum(rbasef2,rcapf2,heightf2,phi1,phi2);

    glTranslatef(0.0, 0.0, heightf2);
    glColor3f(0.0, 0.0, 1.0);
    makeCap(rcapf2,3.14159/2.0-alphaf2,phi1,phi2);
    
    glPopMatrix();

#if 1
    angle += 10.0*3.14/180.0;
    spinangle += 15.0;
#endif
    
    return;
}



- (void)GLresize:sender
{
    glViewport(0,0,(GLint)[self bounds].size.width,(GLint)[self bounds].size.height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-0.25, 0.25, -0.25, 0.25, 1.0, 100.0);
    /* glOrtho(-30.0, 30.0, -30.0, 30.0, -30.0, 30.0); */
    glMatrixMode(GL_MODELVIEW);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    return;
}


void makeFrustum(float rbase, float rcap, float height,
	      float phi1,  float phi2) {
    float phi,delphi,alpha;
    float x,y,z,normx,normy,normz;

    glBegin(GL_QUAD_STRIP);

    if(height > 0.0)
	alpha = atan((rbase-rcap)/height);
    else
	alpha = 0.0;
    
    delphi = 3.14159/18.0;
    for(phi=phi1; phi<phi2; phi+= delphi) {
	x = rbase * cos(phi);
	y = rbase * sin(phi);
	z = 0.0;
	normx = cos(alpha)*cos(phi);
	normy = cos(alpha)*sin(phi);
	normz = sin(alpha);

	glNormal3f(normx,normy,normz);
	glVertex3f(x,y,z);

	x = rcap * cos(phi);
	y = rcap * sin(phi);
	z = height;

	glVertex3f(x,y,z);
    }

    phi = phi2;
    x = rbase * cos(phi);
    y = rbase * sin(phi);
    z = 0.0;
    normx = cos(alpha)*cos(phi);
    normy = cos(alpha)*sin(phi);
    normz = sin(alpha);

    glNormal3f(normx,normy,normz);
    glVertex3f(x,y,z);

    x = rcap * cos(phi);
    y = rcap * sin(phi);
    z = height;

    glVertex3f(x,y,z);
    
    glEnd();
    return;
}


void makeCap(float r, float alpha, float phi1, float phi2) {
    float phi,delphi;
    float theta,deltheta;
    float R,h,x,y,z,normx,normy,normz;

    if(alpha > 0.0) R = r/sin(alpha);
    else return;

    h = R*cos(alpha);

    deltheta = alpha/10.0;
    delphi = (phi2 - phi1)/50.0;

    for(theta=alpha; theta-deltheta>=0.0; theta -= deltheta) {
	glBegin(GL_QUAD_STRIP);
	for(phi=phi1; phi<phi2; phi+= delphi) {
	    x = R*sin(theta)*cos(phi);
	    y = R*sin(theta)*sin(phi);
	    z = R*cos(theta)-h;
	    normx = x/R;
	    normy = y/R;
	    normz = (z+h)/R;

	    glNormal3f(normx,normy,normz);
	    glVertex3f(x,y,z);

	    x = R*sin(theta-deltheta)*cos(phi);
	    y = R*sin(theta-deltheta)*sin(phi);
	    z = R*cos(theta-deltheta)-h;
	    normx = x/R;
	    normy = y/R;
	    normz = (z+h)/R;

	    glNormal3f(normx,normy,normz);
	    glVertex3f(x,y,z);
	}
	phi = phi2;
	x = R*sin(theta)*cos(phi);
	y = R*sin(theta)*sin(phi);
	z = R*cos(theta)-h;
	normx = x/R;
	normy = y/R;
	normz = (z+h)/R;

	glNormal3f(normx,normy,normz);
	glVertex3f(x,y,z);

	x = R*sin(theta-deltheta)*cos(phi);
	y = R*sin(theta-deltheta)*sin(phi);
	z = R*cos(theta-deltheta)-h;
	normx = x/R;
	normy = y/R;
	normz = (z+h)/R;

	glNormal3f(normx,normy,normz);
	glVertex3f(x,y,z);
	glEnd();
    }

    theta = deltheta;
    glBegin(GL_QUAD_STRIP);
    for(phi=phi1; phi<phi2; phi+= delphi) {
	x = R*sin(theta)*cos(phi);
	y = R*sin(theta)*sin(phi);
	z = R*cos(theta)-h;
	normx = x/R;
	normy = y/R;
	normz = (z+h)/R;

	glNormal3f(normx,normy,normz);
	glVertex3f(x,y,z);

	x = R*sin(theta-deltheta)*cos(phi);
	y = R*sin(theta-deltheta)*sin(phi);
	z = R*cos(theta-deltheta)-h;
	normx = x/R;
	normy = y/R;
	normz = (z+h)/R;

	glNormal3f(normx,normy,normz);
	glVertex3f(x,y,z);
    }
    phi = phi2;
    x = R*sin(theta)*cos(phi);
    y = R*sin(theta)*sin(phi);
    z = R*cos(theta)-h;
    normx = x/R;
    normy = y/R;
    normz = (z+h)/R;

    glNormal3f(normx,normy,normz);
    glVertex3f(x,y,z);

    x = R*sin(theta-deltheta)*cos(phi);
    y = R*sin(theta-deltheta)*sin(phi);
    z = R*cos(theta-deltheta)-h;
    normx = x/R;
    normy = y/R;
    normz = (z+h)/R;

    glNormal3f(normx,normy,normz);
    glVertex3f(x,y,z);

    glEnd();
}	

@end

