#import "TextureView.h"

@implementation TextureView

- (void)GLinit:sender
{
    char *exten = (char *) glGetString( GL_EXTENSIONS );
    char *version = (char *) glGetString( GL_VERSION );

    Angle = -20.0;

    /* Check for TexObj stuff */
    if (   strstr( exten, "GL_EXT_texture_object" )
	|| strncmp( version, "1.1", 3 )==0 )
        HaveTexObj = GL_TRUE;
    else
        HaveTexObj = GL_FALSE;

    /* Initialize texture maps */
    [self initTexMaps];

    timer = [NSTimer scheduledTimerWithTimeInterval:(NSTimeInterval)0.025
	             target:self selector:@selector(display)
	             userInfo:nil repeats:YES];

    return;
}
	  

- (void)GLdraw:sender
{
    glDepthFunc(GL_EQUAL);
    /*   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );*/
    glClear( GL_COLOR_BUFFER_BIT );
    
    glColor3f( 1.0, 1.0, 1.0 );
    
    /* draw first polygon */
    glPushMatrix();
    glTranslatef( -1.0, 0.0, 0.0 );
    glRotatef( Angle, 0.0, 0.0, 1.0 );
    if (HaveTexObj) {
	glBindTexture( GL_TEXTURE_2D, TexObj[0] );
    }
    else {
	glCallList( TexObj[0] );
    }
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0, 0.0 );   glVertex2f( -1.0, -1.0 );
    glTexCoord2f( 1.0, 0.0 );   glVertex2f(  1.0, -1.0 );
    glTexCoord2f( 1.0, 1.0 );   glVertex2f(  1.0,  1.0 );
    glTexCoord2f( 0.0, 1.0 );   glVertex2f( -1.0,  1.0 );
    glEnd();
    glPopMatrix();
    
    /* draw second polygon */
    glPushMatrix();
    glTranslatef( 1.0, 0.0, 0.0 );
    glRotatef( Angle-90.0, 0.0, 1.0, 0.0 );
    if (HaveTexObj) {
	glBindTexture( GL_TEXTURE_2D, TexObj[1] );
    }
    else {
	glCallList( TexObj[0] );
    }
    glBegin( GL_POLYGON );
    glTexCoord2f( 0.0, 0.0 );   glVertex2f( -1.0, -1.0 );
    glTexCoord2f( 1.0, 0.0 );   glVertex2f(  1.0, -1.0 );
    glTexCoord2f( 1.0, 1.0 );   glVertex2f(  1.0,  1.0 );
    glTexCoord2f( 0.0, 1.0 );   glVertex2f( -1.0,  1.0 );
    glEnd();
    glPopMatrix();

    Angle += 8.0;
    
    return;
}

- (void)GLresize:sender
{
   glViewport(0, 0, (GLint)[self bounds].size.width, (GLint)[self bounds].size.height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   /*   glOrtho( -3.0, 3.0, -3.0, 3.0, -10.0, 10.0 );*/
   glFrustum( -2.0, 2.0, -2.0, 2.0, 6.0, 20.0 );
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslatef( 0.0, 0.0, -8.0 );

    return;
}


- (void)initTexMaps
{
    static int width=8, height=8;
    static GLubyte tex1[] = {
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 1, 0, 0, 0,
	0, 0, 0, 1, 1, 0, 0, 0,
	0, 0, 0, 0, 1, 0, 0, 0,
	0, 0, 0, 0, 1, 0, 0, 0,
	0, 0, 0, 0, 1, 0, 0, 0,
	0, 0, 0, 1, 1, 1, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0
    };
    
    static GLubyte tex2[] = {
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 2, 2, 0, 0, 0,
	0, 0, 2, 0, 0, 2, 0, 0,
	0, 0, 0, 0, 0, 2, 0, 0,
	0, 0, 0, 0, 2, 0, 0, 0,
	0, 0, 0, 2, 0, 0, 0, 0,
	0, 0, 2, 2, 2, 2, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0
    };
    
    GLubyte tex[64][3];
    GLint i, j;
    
    glDisable( GL_DITHER );
    
    /* Setup texturing */
    glEnable( GL_TEXTURE_2D );
    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST );
    
    
    /* generate texture object IDs */
    if (HaveTexObj) {
	glGenTextures( 2, TexObj );
    }
    else {
	TexObj[0] = glGenLists(2);
	TexObj[1] = TexObj[0]+1;
    }
    
    /* setup first texture object */
    if (HaveTexObj) {
	glBindTexture( GL_TEXTURE_2D, TexObj[0] );
    }
    else {
	glNewList( TexObj[0], GL_COMPILE );
    }
    /* red on white */
    for (i=0;i<height;i++) {
	for (j=0;j<width;j++) {
	    int p = i*width+j;
	    if (tex1[(height-i-1)*width+j]) {
		tex[p][0] = 255;   tex[p][1] = 0;     tex[p][2] = 0;
	    }
	    else {
		tex[p][0] = 255;   tex[p][1] = 255;   tex[p][2] = 255;
	    }
	}
    }
    
    glTexImage2D( GL_TEXTURE_2D, 0, 3, width, height, 0,
		 GL_RGB, GL_UNSIGNED_BYTE, tex );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    if (!HaveTexObj) {
	glEndList();
    }
    /* end of texture object */
    
    
    /* setup second texture object */
    if (HaveTexObj) {
	glBindTexture( GL_TEXTURE_2D, TexObj[1] );
    }
    else {
	glNewList( TexObj[1], GL_COMPILE );
    }
    /* green on blue */
    for (i=0;i<height;i++) {
	for (j=0;j<width;j++) {
	    int p = i*width+j;
	    if (tex2[(height-i-1)*width+j]) {
		tex[p][0] = 0;     tex[p][1] = 255;   tex[p][2] = 0;
	    }
	    else {
		tex[p][0] = 0;     tex[p][1] = 0;     tex[p][2] = 255;
	    }
	}
    }
    glTexImage2D( GL_TEXTURE_2D, 0, 3, width, height, 0,
		 GL_RGB, GL_UNSIGNED_BYTE, tex );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    if (!HaveTexObj) {
	glEndList();
    }
    /* end texture object */
    
}


@end


