#import "MesaView.h"
#import <math.h>
#import <AppKit/NSColor.h>

#ifndef PI
#define PI 3.1415926535
#endif

typedef enum {
    N3D_Perspective,N3D_Orthographic
} N3DProjectionType;

@interface N3DView : MesaView
{
    GLfloat default_ambient[4];   /* RGBA Values of Ambient Light source */
    GLfloat default_diffuse[4];   /* RGBA Values of Diffuse Light source */
    GLfloat default_specular[4];  /* RGBA Values of Specular Light source */
    GLfloat default_position[4];  /* XYZ Values of Light source Position */
    GLfloat default_spotexp[4];

    N3DProjectionType projectionType;
    NSColor *backgroundColor;
    BOOL drawBackground;
	
    struct _projectionRectangle {
	float l,r,t,b,n,f;
    } projectionRectangle;

    float eyePoint[3];
    float viewPoint[3];
    float rollAngle;
}


- initWithFrame:(NSRect)frameRect;
- renderAxes:(double)radius :(double)height;

- setBackgroundColor:(NSColor *)color;
- (NSColor *)backgroundColor;

- setDrawBackgroundColor:(BOOL)flag;
- (BOOL)doesDrawBackgroundColor;

- setProjection:(N3DProjectionType)aProjection;
- (N3DProjectionType)projectionType;

- setClipBoxLeft:(float)left right:(float)right bottom:(float)bottom top:(float)top
	    near:(float)near far:(float)far;

- setClipPlanesNear:(float)aNearPlane far:(float)aFarPlane;

- getProjectionRectangle:(float *)left :(float *)right :(float *)top :(float *)bottom;
- setEyeAt:(float *)fromPoint toward:(float *)toPoint roll:(float)aRollAngle;

- setFieldOfViewByFocalLength:(float)viewAngle;

- useDefaultLighting:(BOOL)flag;

@end
